##############################
## Created by Sandor Laki   ##
## Email: lakis@inf.elte.hu ##
##############################

# An actor class. Base class of all gaming objects. 

import os, pygame,math
from random import randint
import time
from pygame.locals import *

from Globals import *
 
class Actor(pygame.sprite.Sprite):
	"""An actor"""

	RUN = 2001
	WALK = 2002
	STOPPED = 2003

	def __init__(self, image, map):
		pygame.sprite.Sprite.__init__(self)
		self.map = map
		self.state = self.STOPPED
		self.image = image
		self.rect = image.get_rect() 
		self.r_speed = 5
		self.w_speed = 2
		self.x = 10.0
		self.y = 10.0
		self.targetx = 0
		self.targety = 0
		self.dirx = 0.0
		self.diry = 0.0
		self.setLocation( 100, 100 );
		

	def setLocation(self,x,y):
		"""Set location of the actor"""
		self.x = x
		self.y = y		
		self.rect.centerx = int(x)
		self.rect.centery = int(y)
		self.state = self.STOPPED
	
	def goTo(self,x,y,run):
		"""Moving into x,y. If run true, then actors state will be set RUN"""
		self.targetx = x
		self.targety = y
		dirx = float(x)-self.x
		diry = float(y)-self.y
		if dirx == 0 and diry==0:
			self.state = self.STOPPED
			return
		
		norm = math.sqrt(dirx*dirx+diry*diry)
		
		self.dirx = dirx/norm
		self.diry = diry/norm
		
		if run:
			self.state = self.RUN
		else:
			self.state = self.WALK

	def getSpeed(self):
		"""Getting current speed."""
		if self.state == self.RUN:
			return self.r_speed
		else:
			return self.w_speed

	def update(self):
		"""Update sprite"""
		if self.state == self.STOPPED:
			self.handle_stopped()
		else:
			# if RUN or WALK
			# determine new positions on the screen
			newx = int(self.x+self.dirx*self.getSpeed())
			newy = int(self.y+self.diry*self.getSpeed())

			if self.map.isFree(newx/BSIZE,newy/BSIZE):
				dist = math.sqrt((self.x-self.targetx)*(self.x-self.targetx) + (self.y-self.targety)*(self.y-self.targety))
				if dist < self.getSpeed():
					self.state = self.STOPPED
				else:
					self.x += self.dirx*self.getSpeed()
					self.y += self.diry*self.getSpeed()
			else:
				self.state = self.STOPPED
				
			self.rect.centerx = int(self.x)
			self.rect.centery = int(self.y)
			if self.state == self.RUN:
				self.handle_run()
			else:
				self.handle_walk()
		
	def setState(self, state):
		self.state = state
	
	# "pure" virtual functions:
	def handle_stopped(self):
		pass
	
	def handle_run(self):
		pass
		
	def handle_walk(self):
		pass
		
