#/usr/bin/env python 
##############################
## Created by Sandor Laki   ##
## Email: lakis@inf.elte.hu ##
##############################

import os, pygame,math 
from pygame.locals import * 

from Dog import Dog
from Map import Map
from Sheep import Sheep
from Checkpoint import Checkpoint
from Globals import *

class Game:
	"""Game class - Represents a level in this game"""
	def __init__(self,screen,level):
		self.screen = screen
		self.map = Map()
		self.sheeps = []
		self.sheepim = pygame.image.load("./images/sheep.png").convert()
        	self.sheepim.set_colorkey((0xff, 0xff, 0xff), RLEACCEL) 
	 		
		self.dog = Dog(self.map)
		self.rp = pygame.sprite.RenderPlain( (self.dog) )

		self.loadLevel( level )
		self.sp = pygame.sprite.RenderPlain( self.sheeps )
	
	def loadLevel(self, name):
		"""Loading level from a desc file"""
		inp = open( name )
				
		lines = inp.readlines()
		curr = 0
		while curr<len(lines):
			l = lines[curr].rstrip()
			if l=="map:":
				self.map.loadMap("./data/"+lines[curr+1].rstrip())
				curr += 2
			elif l=="start:":
				x = int(lines[curr+1].rstrip())
				y = int(lines[curr+2].rstrip())
				self.dog.setLocation(x,y)
				curr += 3
			elif l=="sheep:":
				x = int(lines[curr+1].rstrip())
				y = int(lines[curr+2].rstrip())
				self.sheeps.append( Sheep(self.map,self.sheepim,self.dog, x, y) )
				curr += 3
			elif l=="checkpoint:":
				top = int(lines[curr+1].rstrip())
				left = int(lines[curr+2].rstrip())
				w = int(lines[curr+3].rstrip())
				h = int(lines[curr+4].rstrip())
			
				self.ch = Checkpoint( self.sheeps, Rect(top,left,w,h) )
				self.ch.activate()
				curr += 5
			else:
				curr += 1			
		inp.close()
	
	def drawIndicators(self, screen):
		"""Show the energy of the dogs"""
		if pygame.font:
		    font = pygame.font.Font(None, 20)
		    text = font.render("Energy:"+str(self.dog.energy), 1, (10, 10, 10))
		    textpos = text.get_rect(centerx=screen.get_width()/2)
		    screen.blit(text, textpos)
		
	
	def run(self):		
		clock = pygame.time.Clock() 
		runstate = 0
		old_time = -1
		while 1:
			# main loop - 2
            		clock.tick(30)
                                 
		        if runstate == 0:
                		if old_time == -1:
                    			old_time = pygame.time.get_ticks()
                    
                		if (pygame.time.get_ticks()-old_time) >= 0: 
                    			old_time = -1
                    			runstate = 1
                    
                		for event in pygame.event.get():
		                	if event.type == QUIT:
                				return -2
					elif event.type == KEYDOWN:
						if event.key == K_ESCAPE:
							return -1# main menu
					elif event.type == MOUSEBUTTONDOWN:
							tx = event.pos[0]
							ty = event.pos[1]
							self.dog.goTo(tx,ty,True)
 			elif runstate == 1:
				for event in pygame.event.get():
		                	if event.type == QUIT:
                				return -2
					elif event.type == KEYDOWN:
						if event.key == K_ESCAPE:
							return -1# main menu
					elif event.type == MOUSEBUTTONDOWN:

							tx = event.pos[0]
							ty = event.pos[1]
							self.dog.goTo(tx,ty,True)
		
				self.dog.update()
				self.sp.update()
				
				self.ch.update()
				self.map.draw( self.screen )
				self.drawIndicators( self.screen )
				self.ch.draw( self.screen )
				self.sp.draw( self.screen )
				self.rp.draw( self.screen )
				pygame.display.flip()
				
				if self.ch.checked:
					return 0
				



